@@ -6,33 +6,45 @@ import os
 import socket as stdlib_socket
 import sys
 import tempfile
+from pathlib import Path
 from socket import AddressFamily, SocketKind
-from typing import TYPE_CHECKING, Any, Callable, List, Tuple, Union
+from typing import TYPE_CHECKING, Union, cast
 
-import attr
+import attrs
 import pytest
 
 from .. import _core, socket as tsocket
-from .._core._tests.tutil import binds_ipv6, creates_ipv6
-from .._socket import _NUMERIC_ONLY, SocketType, _SocketType, _try_sync
+from .._core._tests.tutil import binds_ipv6, can_create_ipv6, creates_ipv6
+from .._socket import _NUMERIC_ONLY, AddressFormat, SocketType, _SocketType, _try_sync
 from ..testing import assert_checkpoints, wait_all_tasks_blocked
 
 if TYPE_CHECKING:
+    from collections.abc import Callable
+
     from typing_extensions import TypeAlias
 
     from .._highlevel_socket import SocketStream
 
-    GaiTuple: TypeAlias = Tuple[
+    GaiTuple: TypeAlias = tuple[
         AddressFamily,
         SocketKind,
         int,
         str,
-        Union[Tuple[str, int], Tuple[str, int, int, int]],
+        Union[tuple[str, int], tuple[str, int, int, int], tuple[int, bytes]],
+    ]
+    GetAddrInfoResponse: TypeAlias = list[GaiTuple]
+    GetAddrInfoArgs: TypeAlias = tuple[
+        Union[str, bytes, None],
+        Union[str, bytes, int, None],
+        int,
+        int,
+        int,
+        int,
     ]
-    GetAddrInfoResponse: TypeAlias = List[GaiTuple]
 else:
     GaiTuple: object
     GetAddrInfoResponse = object
+    GetAddrInfoArgs = object
 
 ################################################################
 # utils
@@ -40,32 +52,75 @@ else:
 
 
 class MonkeypatchedGAI:
-    def __init__(self, orig_getaddrinfo: Callable[..., GetAddrInfoResponse]) -> None:
+    __slots__ = ("_orig_getaddrinfo", "_responses", "record")
+
+    def __init__(
+        self,
+        orig_getaddrinfo: Callable[
+            [str | bytes | None, str | bytes | int | None, int, int, int, int],
+            GetAddrInfoResponse,
+        ],
+    ) -> None:
         self._orig_getaddrinfo = orig_getaddrinfo
-        self._responses: dict[tuple[Any, ...], GetAddrInfoResponse | str] = {}
-        self.record: list[tuple[Any, ...]] = []
+        self._responses: dict[
+            GetAddrInfoArgs,
+            GetAddrInfoResponse | str,
+        ] = {}
+        self.record: list[GetAddrInfoArgs] = []
 
     # get a normalized getaddrinfo argument tuple
-    def _frozenbind(self, *args: Any, **kwargs: Any) -> tuple[Any, ...]:
+    def _frozenbind(
+        self,
+        host: str | bytes | None,
+        port: str | bytes | int | None,
+        family: int = 0,
+        type: int = 0,
+        proto: int = 0,
+        flags: int = 0,
+    ) -> GetAddrInfoArgs:
         sig = inspect.signature(self._orig_getaddrinfo)
-        bound = sig.bind(*args, **kwargs)
+        bound = sig.bind(host, port, family=family, type=type, proto=proto, flags=flags)
         bound.apply_defaults()
         frozenbound = bound.args
         assert not bound.kwargs
         return frozenbound
 
     def set(
-        self, response: GetAddrInfoResponse | str, *args: Any, **kwargs: Any
+        self,
+        response: GetAddrInfoResponse | str,
+        host: str | bytes | None,
+        port: str | bytes | int | None,
+        family: int = 0,
+        type: int = 0,
+        proto: int = 0,
+        flags: int = 0,
     ) -> None:
-        self._responses[self._frozenbind(*args, **kwargs)] = response
+        self._responses[
+            self._frozenbind(
+                host,
+                port,
+                family=family,
+                type=type,
+                proto=proto,
+                flags=flags,
+            )
+        ] = response
 
-    def getaddrinfo(self, *args: Any, **kwargs: Any) -> GetAddrInfoResponse | str:
-        bound = self._frozenbind(*args, **kwargs)
+    def getaddrinfo(
+        self,
+        host: str | bytes | None,
+        port: str | bytes | int | None,
+        family: int = 0,
+        type: int = 0,
+        proto: int = 0,
+        flags: int = 0,
+    ) -> GetAddrInfoResponse | str:
+        bound = self._frozenbind(host, port, family, type, proto, flags)
         self.record.append(bound)
         if bound in self._responses:
             return self._responses[bound]
-        elif bound[-1] & stdlib_socket.AI_NUMERICHOST:
-            return self._orig_getaddrinfo(*args, **kwargs)
+        elif flags & stdlib_socket.AI_NUMERICHOST:
+            return self._orig_getaddrinfo(host, port, family, type, proto, flags)
         else:
             raise RuntimeError(f"gai called with unexpected arguments {bound}")
 
@@ -131,11 +186,11 @@ async def test_getaddrinfo(monkeygai: MonkeypatchedGAI) -> None:
         ) -> tuple[
             AddressFamily,
             SocketKind,
-            tuple[str, int] | tuple[str, int, int] | tuple[str, int, int, int],
+            tuple[str, int] | tuple[str, int, int, int] | tuple[int, bytes],
         ]:
             # (family, type, proto, canonname, sockaddr)
-            family, type, proto, canonname, sockaddr = gai_tup
-            return (family, type, sockaddr)
+            family, type_, _proto, _canonname, sockaddr = gai_tup
+            return (family, type_, sockaddr)
 
         def filtered(
             gai_list: GetAddrInfoResponse,
@@ -143,7 +198,7 @@ async def test_getaddrinfo(monkeygai: MonkeypatchedGAI) -> None:
             tuple[
                 AddressFamily,
                 SocketKind,
-                tuple[str, int] | tuple[str, int, int] | tuple[str, int, int, int],
+                tuple[str, int] | tuple[str, int, int, int] | tuple[int, bytes],
             ]
         ]:
             return [interesting_fields(gai_tup) for gai_tup in gai_list]
@@ -321,10 +376,12 @@ async def test_sniff_sockopts() -> None:
     from socket import AF_INET, AF_INET6, SOCK_DGRAM, SOCK_STREAM
 
     # generate the combinations of families/types we're testing:
-    sockets = []
-    for family in [AF_INET, AF_INET6]:
-        for type in [SOCK_DGRAM, SOCK_STREAM]:
-            sockets.append(stdlib_socket.socket(family, type))
+    families = (AF_INET, AF_INET6) if can_create_ipv6 else (AF_INET,)
+    sockets = [
+        stdlib_socket.socket(family, type_)
+        for family in families
+        for type_ in [SOCK_DGRAM, SOCK_STREAM]
+    ]
     for socket in sockets:
         # regular Trio socket constructor
         tsocket_socket = tsocket.socket(fileno=socket.fileno())
@@ -410,7 +467,14 @@ def setsockopt_tests(sock: SocketType | SocketStream) -> None:
     # specifying optlen. Not supported on pypy, and I couldn't find
     # valid calls on darwin or win32.
     if hasattr(tsocket, "SO_BINDTODEVICE"):
-        sock.setsockopt(tsocket.SOL_SOCKET, tsocket.SO_BINDTODEVICE, None, 0)
+        try:
+            sock.setsockopt(tsocket.SOL_SOCKET, tsocket.SO_BINDTODEVICE, None, 0)
+        except OSError as e:
+            # some versions of Python have the attribute yet can run on platforms
+            # that do not support it. For instance, MacOS 15 gained support for
+            # SO_BINDTODEVICE and CPython 3.13.1 was built on it (presumably), but
+            # our CI runners ran MacOS 14 and so failed.
+            assert e.errno == 42  # noqa: PT017
 
     # specifying value
     sock.setsockopt(tsocket.IPPROTO_TCP, tsocket.TCP_NODELAY, False)
@@ -471,7 +535,8 @@ async def test_SocketType_shutdown() -> None:
     ],
 )
 async def test_SocketType_simple_server(
-    address: str, socket_type: AddressFamily
+    address: str,
+    socket_type: AddressFamily,
 ) -> None:
     # listen, bind, accept, connect, getpeername, getsockname
     listener = tsocket.socket(socket_type)
@@ -512,12 +577,12 @@ def gai_without_v4mapped_is_buggy() -> bool:  # pragma: no cover
         return True
 
 
-@attr.s
+@attrs.define(slots=False)
 class Addresses:
-    bind_all: str = attr.ib()
-    localhost: str = attr.ib()
-    arbitrary: str = attr.ib()
-    broadcast: str = attr.ib()
+    bind_all: str
+    localhost: str
+    arbitrary: str
+    broadcast: str
 
 
 # Direct thorough tests of the implicit resolver helpers
@@ -555,7 +620,8 @@ async def test_SocketType_resolve(socket_type: AddressFamily, addrs: Addresses)
         return addr
 
     def assert_eq(
-        actual: tuple[str | int, ...], expected: tuple[str | int, ...]
+        actual: tuple[str | int, ...],
+        expected: tuple[str | int, ...],
     ) -> None:
         assert pad(expected) == pad(actual)
 
@@ -580,17 +646,21 @@ async def test_SocketType_resolve(socket_type: AddressFamily, addrs: Addresses)
         for local in [False, True]:
 
             async def res(
-                args: tuple[str, int]
-                | tuple[str, int, int]
-                | tuple[str, int, int, int]
-                | tuple[str, str]
-                | tuple[str, str, int]
-                | tuple[str, str, int, int]
-            ) -> Any:
-                return await sock._resolve_address_nocp(
+                args: (
+                    tuple[str, int]
+                    | tuple[str, int, int]
+                    | tuple[str, int, int, int]
+                    | tuple[str, str]
+                    | tuple[str, str, int]
+                    | tuple[str, str, int, int]
+                ),
+            ) -> tuple[str | int, ...]:
+                value = await sock._resolve_address_nocp(
                     args,
                     local=local,  # noqa: B023  # local is not bound in function definition
                 )
+                assert isinstance(value, tuple)
+                return cast("tuple[Union[str, int], ...]", value)
 
             assert_eq(await res((addrs.arbitrary, "http")), (addrs.arbitrary, 80))
             if v6:
@@ -624,8 +694,8 @@ async def test_SocketType_resolve(socket_type: AddressFamily, addrs: Addresses)
                 sock.setsockopt(tsocket.IPPROTO_IPV6, tsocket.IPV6_V6ONLY, True)
                 with pytest.raises(tsocket.gaierror) as excinfo:
                     await res(("1.2.3.4", 80))
-                # Windows, macOS
-                expected_errnos = {tsocket.EAI_NONAME}
+                # Windows, macOS, musl/Linux
+                expected_errnos = {tsocket.EAI_NONAME, tsocket.EAI_NODATA}
                 # Linux
                 if hasattr(tsocket, "EAI_ADDRFAMILY"):
                     expected_errnos.add(tsocket.EAI_ADDRFAMILY)
@@ -636,7 +706,8 @@ async def test_SocketType_resolve(socket_type: AddressFamily, addrs: Addresses)
             # smoke test the basic functionality...
             try:
                 netlink_sock = tsocket.socket(
-                    family=tsocket.AF_NETLINK, type=tsocket.SOCK_DGRAM
+                    family=tsocket.AF_NETLINK,
+                    type=tsocket.SOCK_DGRAM,
                 )
             except (AttributeError, OSError):
                 pass
@@ -653,7 +724,7 @@ async def test_SocketType_resolve(socket_type: AddressFamily, addrs: Addresses)
                 await res("1.2.3.4")  # type: ignore[arg-type]
             with pytest.raises(ValueError, match=address):
                 await res(("1.2.3.4",))  # type: ignore[arg-type]
-            with pytest.raises(  # noqa: PT012
+            with pytest.raises(
                 ValueError,
                 match=address,
             ):
@@ -784,15 +855,20 @@ async def test_SocketType_connect_paths() -> None:
             # nose -- and then swap it back out again before we hit
             # wait_socket_writable, which insists on a real socket.
             class CancelSocket(stdlib_socket.socket):
-                def connect(self, *args: Any, **kwargs: Any) -> None:
+                def connect(
+                    self,
+                    address: AddressFormat,
+                ) -> None:
                     # accessing private method only available in _SocketType
                     assert isinstance(sock, _SocketType)
 
                     cancel_scope.cancel()
                     sock._sock = stdlib_socket.fromfd(
-                        self.detach(), self.family, self.type
+                        self.detach(),
+                        self.family,
+                        self.type,
                     )
-                    sock._sock.connect(*args, **kwargs)
+                    sock._sock.connect(address)
                     # If connect *doesn't* raise, then pretend it did
                     raise BlockingIOError  # pragma: no cover
 
@@ -839,13 +915,17 @@ async def test_resolve_address_exception_in_connect_closes_socket() -> None:
         with tsocket.socket() as sock:
 
             async def _resolve_address_nocp(
-                self: Any, *args: Any, **kwargs: Any
+                address: AddressFormat,
+                *,
+                local: bool,
             ) -> None:
+                assert address == ""
+                assert not local
                 cancel_scope.cancel()
                 await _core.checkpoint()
 
             assert isinstance(sock, _SocketType)
-            sock._resolve_address_nocp = _resolve_address_nocp  # type: ignore[method-assign, assignment]
+            sock._resolve_address_nocp = _resolve_address_nocp  # type: ignore[method-assign]
             with assert_checkpoints():
                 with pytest.raises(_core.Cancelled):
                     await sock.connect("")
@@ -977,12 +1057,20 @@ async def test_custom_hostname_resolver(monkeygai: MonkeypatchedGAI) -> None:
     # This intentionally breaks the signatures used in HostnameResolver
     class CustomResolver:
         async def getaddrinfo(
-            self, host: str, port: str, family: int, type: int, proto: int, flags: int
+            self,
+            host: str,
+            port: str,
+            family: int,
+            type: int,
+            proto: int,
+            flags: int,
         ) -> tuple[str, str, str, int, int, int, int]:
             return ("custom_gai", host, port, family, type, proto, flags)
 
         async def getnameinfo(
-            self, sockaddr: tuple[str, int] | tuple[str, int, int, int], flags: int
+            self,
+            sockaddr: tuple[str, int] | tuple[str, int, int, int],
+            flags: int,
         ) -> tuple[str, tuple[str, int] | tuple[str, int, int, int], int]:
             return ("custom_gni", sockaddr, flags)
 
@@ -1029,7 +1117,10 @@ async def test_custom_hostname_resolver(monkeygai: MonkeypatchedGAI) -> None:
 async def test_custom_socket_factory() -> None:
     class CustomSocketFactory:
         def socket(
-            self, family: AddressFamily, type: SocketKind, proto: int
+            self,
+            family: AddressFamily,
+            type: SocketKind,
+            proto: int,
         ) -> tuple[str, AddressFamily, SocketKind, int]:
             return ("hi", family, type, proto)
 
@@ -1055,7 +1146,7 @@ async def test_custom_socket_factory() -> None:
     assert tsocket.set_custom_socket_factory(None) is csf
 
 
-async def test_SocketType_is_abstract() -> None:
+def test_SocketType_is_abstract() -> None:
     with pytest.raises(TypeError):
         tsocket.SocketType()
 
@@ -1065,7 +1156,7 @@ async def test_unix_domain_socket() -> None:
     # Bind has a special branch to use a thread, since it has to do filesystem
     # traversal. Maybe connect should too? Not sure.
 
-    async def check_AF_UNIX(path: str | bytes) -> None:
+    async def check_AF_UNIX(path: str | bytes | os.PathLike[str]) -> None:
         with tsocket.socket(family=tsocket.AF_UNIX) as lsock:
             await lsock.bind(path)
             lsock.listen(10)
@@ -1079,8 +1170,11 @@ async def test_unix_domain_socket() -> None:
     # Can't use tmpdir fixture, because we can exceed the maximum AF_UNIX path
     # length on macOS.
     with tempfile.TemporaryDirectory() as tmpdir:
-        path = f"{tmpdir}/sock"
-        await check_AF_UNIX(path)
+        # Test passing various supported types as path
+        # Must use different filenames to prevent "address already in use"
+        await check_AF_UNIX(f"{tmpdir}/sock")
+        await check_AF_UNIX(Path(f"{tmpdir}/sock1"))
+        await check_AF_UNIX(os.fsencode(f"{tmpdir}/sock2"))
 
     try:
         cookie = os.urandom(20).hex().encode("ascii")
